# Site components

So a site component is basically just a more complex component like in [more complex component](components.md##%20More%20complex%20components)

Normally a site component don't have any properties from `MyProps` only from the `RootState`

It is also a good idea that every site component has it's own data section inside the `RootState` e.g. rootState.doExerciseState for the doExercise component

```ts
const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    data: rootState.doExerciseState.data //just an example
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //actions
}, dispatch)

interface MatchedProps {
  readonly urlParam: string
}

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class doExerciseSite extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  async componentWillMount(): Promise<void> {
    const urlParam = this.props.match.params.urlParam
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(doExerciseSite))
```

So it is needed that you call `withRouter` if you have any url params you want to access (even if you don't have any it is recommended that you write this in case you later forget it try to find the error for hours...[happened already])


And we are finished ... except that we need to add the new component as a side to the app

## Adding the new site component

Inside the `frontend/src/app.tsx` all sites are specified

We use react router and the `Route` elements to render the correct (matching) component depending on the url

So add a new constant in `frontend/src/constants.ts` and specify the path for the new site

You can use `:urlParam` inside path e.g. `/mysite/:urlParam` or `urlParam?` (for optional params)
to add an url parameter

Then go back end add a new `Route` component

```xml
<Route exact path={constants.path} render={(prop) =>
  <PageContentWrapper>
    <DoExerciseSite/>
  </PageContentWrapper>
}/>
```

*What does the exact means? Don't know, look at the docs for react router, might change with versions*

As you might see other sites have a *headerBar*

The website has a site-header bar (where the menu, user icon is located) and some sites have also a *headerBar* under the site-header bar for some actions e.g. save, delete, ...

## Header bar

As described above some sites also have a `header bar` with actions for the currently active site

A `header bar` is a component normally located next to the site component (in the file tree)

The name is `headerBarContent`, content because we use a generic component called 
`CommonHeaderBar` to enable some actions e.g. collapsing all `header bars`

An example header bar could look like this

```ts
export interface MyProps {
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    isHeaderBarDisabled: rootState.doExerciseState.isHeaderBarDisabled
    isLoading: rootState.doExerciseState.isLoading
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
}, dispatch)

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class HeaderBar extends React.Component<Props, any> {

  render(): JSX.Element {

    const leftArea = (
      <div></div>
    )

    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea}
                                    disabled={this.props.isLoading ||this.props.isHeaderBarDisabled}/>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(HeaderBar)
```

The `CommonHeaderBarContentWrapper` is a helper component to generalize the appearing of the header bars (we have left, center, right areas, can be disabled, ...)

Actually just a normal component ;)


```xml

import DoExerciseSite from './components/sites/DoExerciseSite/DoExerciseSite'
import DoExerciseSiteHeaderBarContent from './components/sites/DoExerciseSite/headerBarContent'

...

<Route exact path={constants.path} render={(prop) =>
  <PageContentWrapper
    headerBar={
      <CommonHeaderBar>
        <DoExerciseSiteHeaderBarContent/>
      </CommonHeaderBar>
    }
  >
        <DoExerciseSite/>
  </PageContentWrapper>
}/>
```

## Adding links

If we want to add a link to our new site add

```ts
import * as React from "react";
import {Link} from "react-router-dom";
import {Icon} from "semantic-ui-react";
import * as constants from "../../constants";
import {TranslateIds} from "../../../i18n/ids";
import {translate} from "../../../i18n/translationHelper";

<Link to={constants.path}}>
  <Icon size="huge" name='flag'/>
  <span>
    {
      translate(TranslateIds.myNewSite)
    }
  </span>
</Link>
```

to some component


## Redux Actions

For every site components it's good to have an action

`loadXXXSite` like [redux multiple async actions](redux.md##%20Multiple%20async%20actions) which will load all data for the site.

The first action inside the loading action should be to clear old data, so calling the `reset` action "of the reducer"

## Common patterns

### Single panel site


```ts
import SinglePanelSiteWrapper from ...
import Spinner from ...
import MaterialSearchInput from ...

render(): JSX.Element {

   return (
      <SinglePanelSiteWrapper>
        {
          this.props.isLoading &&
          <Spinner/>
        }

        <div className="view-padding">

          <div className="view-head-line">

            <div className="view-caption">
              <h2>
                {
                  translate(TranslateIds.information_pl)
                }
              </h2>
            </div>

            <div className="view-options">
              <MaterialSearchInput/>
            </div>
          </div>

          <div className="view-content">
          {
            //content
          }
          </div
      </SinglePanelSiteWrapper>
   )
}
```


### Multiple panel site

```ts
render(): JSX.Element {

   return (
      <div className="multiple-single-panels">
        <FirstPanel />

        <SecondPanel />
      </div>
   )
}
```

## Hints

Some site components (e.g. do exercise site) are long living components.

So it's more likely that there will be a login timeout and the user must re-login.

Because when the login is displayed all other components are destroyed they need to be re-created when the login is finished.

Normally all components use the `componentWillMount` function to reset the site dat and get the new data from the backend.
When doing a re-login this is not really useful because e.g. the editor content would be overwritten with the content at backend...

There is a helper `LoginTimeoutHelper` which stores some data for re-login.

Please see the do exercise site `componentWillMount` function one how to use it (to use the helper for your own component you probably need to add some properties to the helper).